package com.sun.xml.internal.ws.policy.jaxws;

import com.sun.xml.internal.ws.api.policy.AlternativeSelector;
import com.sun.xml.internal.ws.policy.EffectivePolicyModifier;
import java.util.Iterator;
import javax.xml.ws.WebServiceException;
import com.sun.xml.internal.ws.policy.PolicyException;
import com.sun.xml.internal.ws.resources.PolicyMessages;
import com.sun.xml.internal.ws.policy.spi.PolicyAssertionValidator;
import com.sun.xml.internal.ws.policy.PolicyAssertion;
import com.sun.xml.internal.ws.policy.AssertionSet;
import com.sun.xml.internal.ws.policy.Policy;
import com.sun.xml.internal.ws.api.policy.ValidationProcessor;
import com.sun.xml.internal.ws.policy.PolicyMap;
import com.sun.xml.internal.ws.api.policy.PolicyResolver;

public class DefaultPolicyResolver implements PolicyResolver
{
    public DefaultPolicyResolver() {
    }
    
    public PolicyMap resolve(PolicyResolver.ServerContext context) {
        PolicyMap map = context.getPolicyMap();
        if (map != null) {
            this.validateServerPolicyMap(map);
            return map;
        }
        return map;
    }
    
    public PolicyMap resolve(PolicyResolver.ClientContext context) {
        PolicyMap map = context.getPolicyMap();
        if (map != null) {
            map = this.doAlternativeSelection(map);
            return map;
        }
        return map;
    }
    
    private void validateServerPolicyMap(PolicyMap policyMap) {
        try {
            ValidationProcessor validationProcessor = ValidationProcessor.getInstance();
            for (Policy policy : policyMap) {
                for (AssertionSet assertionSet : policy) {
                    for (PolicyAssertion assertion : assertionSet) {
                        PolicyAssertionValidator.Fitness validationResult = validationProcessor.validateServerSide(assertion);
                        if (validationResult != PolicyAssertionValidator.Fitness.SUPPORTED) {
                            throw new PolicyException(PolicyMessages.WSP_1015_SERVER_SIDE_ASSERTION_VALIDATION_FAILED(assertion.getName(), validationResult));
                        }
                        continue;
                    }
                }
            }
        }
        catch (PolicyException e) {
            throw new WebServiceException(e);
        }
    }
    
    private PolicyMap doAlternativeSelection(PolicyMap policyMap) {
        EffectivePolicyModifier modifier = EffectivePolicyModifier.createEffectivePolicyModifier();
        modifier.connect(policyMap);
        try {
            AlternativeSelector.doSelection(modifier);
        }
        catch (PolicyException e) {
            throw new WebServiceException(e);
        }
        return policyMap;
    }
}
